<?php
/**
 * Zen
 *
 * @author    Zen
 * @copyright Copyright (c) 2022 Zen
 * @license   https://opensource.org/licenses/AFL-3.0  Academic Free License ("AFL") v. 3.0
 *
 * https://www.zen.com/
 */

use Zen\Payment\Api;
use Zen\Payment\Util;

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once(_PS_MODULE_DIR_ . 'zencom/libraries/payment-core/vendor/autoload.php');


/**
 * Class Zen
 *
 * @property string tab
 * @property array ps_versions_compliancy
 * @property bool currencies
 * @property int need_instance
 * @property string author
 * @property string name
 * @property string platformName
 * @property string version
 * @property string currencies_mode
 * @property string displayName
 * @property string description
 * @property string confirm_uninstall
 */
class Zencom extends PaymentModule
{

    /**
     * Sets the Information for the Module manager
     * Also creates an instance of this class
     */
    public function __construct()
    {
        $this->name = 'zencom';
        $this->platformName = 'prestashop_1.7';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.7';
        $this->author = 'Zen';
        $this->need_instance = 1;
        $this->currencies = true;
        $this->currencies_mode = 'checkbox';
        $this->ps_versions_compliancy = [
            'min' => '1.7',
            'max' => '1.7',
        ];
        $this->is_eu_compatible = 1;
        $this->module_key = 'ffc7f50d8b9c2970c145fb6ac764d24a';

        parent::__construct();

        $this->displayName = $this->l('Zen');
        $this->description = $this->l('Visa, MasterCard, BLIK ect.');
        $this->confirm_uninstall = $this->l('Are you sure you want to uninstall Zen module?');
    }

    /**
     * @param $name
     * @param $type
     *
     * @return string
     */
    public static function buildTemplatePath($name, $type)
    {
        return 'module:zencom/views/templates/' . $type . '/' . $name . '.tpl';
    }

    /**
     * This function installs the Zen Module
     *
     * @return boolean
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function install()
    {

        if (!(
            parent::install()
            && $this->registerHook('paymentOptions')
            && $this->registerHook('orderConfirmation')
            && $this->registerHook('paymentReturn')
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('displayHeader')
            && Configuration::updateValue('ZEN_TERMINAL_ID', '')
            && Configuration::updateValue('ZEN_PAYWALL_SECRET', '')
            && Configuration::updateValue('ZEN_IPN_SECRET', '')
            && Configuration::updateValue('ZEN_API_KEY', '')
            && Configuration::updateValue('ZEN_PAYMENT_TITLE', '')
            && Configuration::updateValue('ZEN_SANDBOX', 0)
        )) {
            return false;
        }

        if (Validate::isInt(Configuration::get('PAYMENT_ZEN_NEW_STATUS'))
            xor (
            Validate::isLoadedObject(new OrderState(Configuration::get('PAYMENT_ZEN_NEW_STATUS')))
            )
        ) {
            $orderStateNew = new OrderState();

            $missingLang = true;

            $langs = [
                'en' => 'Payment Zen: awaiting for confirmation',
                'pl' => 'Płatność Zen: oczekuje na potwierdzenie',
            ];

            foreach ($langs as $lang => $message) {
                $langId = Language::getIdByIso($lang);
                if (isset($langId)) {
                    $orderStateNew->name[$langId] = $message;
                    $missingLang = false;
                }
            }

            if ($missingLang) {
                $langId = (int)$this->context->language->id;
                $orderStateNew->name[$langId] = $langs['en'];
            }

            $orderStateNew->send_email = false;
            $orderStateNew->invoice = false;
            $orderStateNew->unremovable = false;
            $orderStateNew->color = "black";

            if (!$orderStateNew->add()) {
                $this->_errors[] = $this->l('There was an Error installing the module. Cannot add new order state.');

                return false;
            }
            if (!Configuration::updateValue('PAYMENT_ZEN_NEW_STATUS', $orderStateNew->id)) {
                $this->_errors[] = $this->l('There was an Error installing the module. Cannot update new order state.');

                return false;
            }

            copy(
                _PS_MODULE_DIR_
                . $this->name
                . '/views/img/zen-icon.png',
                _PS_IMG_DIR_
                . 'os/'
                . $orderStateNew->id
                . '.png'
            );
        }

        return true;
    }

    /**
     * This function uninstalls the Zen Module
     *
     * @return boolean
     */
    public function uninstall()
    {
        return Configuration::deleteByName('ZEN_TERMINAL_ID')
            && Configuration::deleteByName('ZEN_PAYWALL_SECRET')
            && Configuration::deleteByName('ZEN_IPN_SECRET')
            && Configuration::deleteByName('ZEN_API_KEY')
            && Configuration::deleteByName('ZEN_PAYMENT_TITLE')
            && Configuration::deleteByName('ZEN_SANDBOX')
            && parent::uninstall();
    }

    /**
     * Display configuration form.
     *
     * @return mixed
     */
    public function getContent()
    {

        $msg = '';

        if (Tools::isSubmit('submitZen')) {
            if ($this->saveConfiguration()) {
                $msg = [
                    'type' => 'success',
                    'message' => $this->l('Configuration updated successfully'),
                ];
            } else {
                $msg = [
                    'type' => 'error',
                    'message' => $this->l('There was an error saving your configuration'),
                ];
            }
        }

        $this->context->smarty->assign([
            'zen_form' =>
                './index.php?tab=AdminModules&configure=zen&token='
                . Tools::getAdminTokenLite('AdminModules')
                . '&tab_module='
                . $this->tab
                . '&module_name=zen',
            'zen_terminal_id' => Configuration::get('ZEN_TERMINAL_ID'),
            'zen_paywall_secret' => Configuration::get('ZEN_PAYWALL_SECRET'),
            'zen_ipn_secret' => Configuration::get('ZEN_IPN_SECRET'),
            'zen_api_key' => Configuration::get('ZEN_API_KEY'),
            'zen_payment_title' => Configuration::get('ZEN_PAYMENT_TITLE'),
            'zen_msg' => $msg,
            'zen_msg_on' => $this->l('On'),
            'zen_msg_off' => $this->l('Off'),
            'zen_sandbox' => Configuration::get('ZEN_SANDBOX'),
            'zen_payment_title_default' => $this->getPaymentTitleDefault(),
        ]);

        return $this->fetchTemplate('admin.tpl');
    }

    /**
     * @return bool
     */
    private function saveConfiguration()
    {
        Configuration::updateValue('ZEN_TERMINAL_ID', pSQL(Tools::getValue('zen_terminal_id')));
        Configuration::updateValue('ZEN_PAYWALL_SECRET', pSQL(Tools::getValue('zen_paywall_secret')));
        Configuration::updateValue('ZEN_IPN_SECRET', pSQL(Tools::getValue('zen_ipn_secret')));
        Configuration::updateValue('ZEN_API_KEY', pSQL(Tools::getValue('zen_api_key')));
        Configuration::updateValue('ZEN_PAYMENT_TITLE', pSQL(Tools::getValue('zen_payment_title')));
        Configuration::updateValue('ZEN_SANDBOX', pSQL(Tools::getValue('zen_sandbox')));

        return true;
    }

    /**
     * @return string
     */
    private function getPaymentTitleDefault()
    {
        return 'Cards, Bank transfers, BLIK, paysafecard, PayPal with';
    }

    /**
     * @param string $name
     *
     * @return mixed
     */
    public function fetchTemplate($name)
    {
        return $this->display(__FILE__, $name);
    }

    /**
     * Hook that run on mobile version of Prestashop.
     */
    public function hookDisplayMobileHeader()
    {
        $this->hookHeader();
    }

    /**
     * Hook that adds css for old versions of Prestashop.
     */
    public function hookHeader()
    {
        $this->context->controller->addCSS(($this->_path) . 'views/css/zen-legacy-front.min.css', 'all');
    }

    /**
     * return string
     */
    public function hookBackOfficeHeader()
    {

        /**
         * @var string
         */
        $orderId = Tools::getValue('id_order');
        $canRefund = true;

        if ($orderId === false
            || !Configuration::get('ZEN_API_KEY')) {
            $canRefund = false;
        }

        $order = new Order($orderId);

        $orderPayment = null;

        if (isset($order->getOrderPaymentCollection()->getResults()[0])) {
            $orderPayment = $order->getOrderPaymentCollection()->getResults()[0];
        }

        $orderCurrentState = $order->getCurrentState();

        if ($order->module !== $this->name
            || (!isset($orderPayment->transaction_id)
                || !$orderPayment->transaction_id)
            || $orderCurrentState === _PS_OS_CANCELED_
            || $orderCurrentState === _PS_OS_REFUND_
        ) {
            $canRefund = false;
        }

        $refundType = Tools::getValue('zen_refund_type');

        $refundAmount = $refundType === 'full'
            ? $order->total_paid
            : Tools::getValue('zen_refund_amount');

        $refundAmount = (float)$refundAmount;

        $this->context->smarty->assign('zen_refund_amount', $refundAmount);

        if (isset($order) && is_object($order)) {
            $this->context->smarty->assign('zen_refund_full_amount', round($order->total_paid, 2));
        }

        $this->context->smarty->assign('zen_refund_type', $refundType);
        $this->context->smarty->assign('show_refund', $canRefund);

        $refundErrors = [];
        $currencyInfo = '';

        if ($canRefund && empty($refundErrors) && Tools::getValue('submitZenPaymentRefund')) {
            if ($refundAmount > $order->total_paid) {
                $refundErrors[] = $this->l(
                    'The refund amount you entered is greater than paid amount.'
                );
            }

            if (empty($refundErrors)) {
                $api = new Api(
                    Configuration::get('ZEN_API_KEY'),
                    $this->getEnvironment()
                );

                $currencyInfo = Currency::getCurrency($order->id_currency);

                $refund = $api->createRefund(
                    $refundAmount,
                    $orderPayment->transaction_id,
                    $currencyInfo['iso_code'],
                    $orderId
                );

                if (!$refund['success']) {
                    if (isset($refund['data']['zenError']['message']) && $refund['data']['zenError']['message']) {
                        $refundErrors[] = $this->l('Refund error:')
                            . ' '
                            . explode(';', $refund['data']['zenError']['message'])[0];
                    } else {
                        $refundErrors[] = $this->l('Refund error.');
                    }
                }
            }
        }

        if (isset($refund) && empty($refundErrors)) {
            $msg = $this->l('Refund for amount:')
                . ' '
                . $refundAmount
                . ' '
                . $currencyInfo['iso_code']
                . ' '
                . $this->l('with Zen UUID:')
                . ' '
                . $refund['body']['id']
                . ' '
                . $this->l('has been requested and waiting for completion.');

            $this->context->smarty->assign('zen_refund_message', $msg);
        }

        $this->context->smarty->assign('zen_refund_errors', $refundErrors);
        $this->context->smarty->assign('ps_version', (int)str_replace('.', '', _PS_VERSION_));

        return $this->fetchTemplate('/views/templates/admin/refunds.tpl');
    }

    /**
     * @return string
     */
    private static function getEnvironment()
    {
        return Configuration::get('ZEN_SANDBOX') ?
            Util::ENVIRONMENT_SANDBOX :
            Util::ENVIRONMENT_PRODUCTION;
    }

    /**
     * Hook that adds css and js in admin panel.
     */
    public function hookBackOfficeFooter()
    {
        echo '<link type="text/css" rel="stylesheet" href="' . $this->_path . 'views/css/zen-admin.min.css">';
        echo '<link type="text/css" rel="stylesheet" href="' . $this->_path . 'views/css/font-awesome.min.css">';

        echo '<script src="' . $this->_path . 'views/js/zen-legacy-admin.js"></script>';
    }

    /**
     * @param array $params
     *
     * @return array|bool
     * @throws SmartyException
     */
    public function hookPaymentOptions($params)
    {

        if (!Configuration::get('ZEN_TERMINAL_ID')
            || !Configuration::get('ZEN_PAYWALL_SECRET')
            || !Configuration::get('ZEN_IPN_SECRET')
            || !Configuration::get('ZEN_API_KEY')
            || !$this->active) {
            return false;
        }

        $paymentOptionZen = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();

        return [
            $paymentOptionZen->setCallToActionText(
                Configuration::get('ZEN_PAYMENT_TITLE')
                    ?: $this->getPaymentTitleDefault()
            )
                ->setModuleName($this->name)
                ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/views/img/logo.svg'))
                ->setAction($this->context->link->getModuleLink('zencom', 'payment')),
        ];
    }

    /**
     * @return null
     */
    public function hookPaymentReturn()
    {
        return null;
    }
}
